@@ -1,23 +1,20 @@
 /*
-    $Id: solaris.c,v 1.11 2006/03/17 19:06:51 rocky Exp $
+  Copyright (C) 2002, 2003, 2004, 2005, 2006, 2009, 2011
+  2012 Rocky Bernstein <rocky@gnu.org>
+  Copyright (C) 2001 Herbert Valerio Riedel <hvr@gnu.org>
 
-    Copyright (C) 2001 Herbert Valerio Riedel <hvr@gnu.org>
-    Copyright (C) 2002, 2003, 2004, 2005, 2006
-    Rocky Bernstein <rocky@panix.com>
+  This program is free software: you can redistribute it and/or modify
+  it under the terms of the GNU General Public License as published by
+  the Free Software Foundation, either version 3 of the License, or
+  (at your option) any later version.
 
-    This program is free software; you can redistribute it and/or modify
-    it under the terms of the GNU General Public License as published by
-    the Free Software Foundation; either version 2 of the License, or
-    (at your option) any later version.
+  This program is distributed in the hope that it will be useful,
+  but WITHOUT ANY WARRANTY; without even the implied warranty of
+  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+  GNU General Public License for more details.
 
-    This program is distributed in the hope that it will be useful,
-    but WITHOUT ANY WARRANTY; without even the implied warranty of
-    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-    GNU General Public License for more details.
-
-    You should have received a copy of the GNU General Public License
-    along with this program; if not, write to the Free Software
-    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+  You should have received a copy of the GNU General Public License
+  along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */
 
 #ifdef HAVE_CONFIG_H
@@ -39,7 +36,7 @@
 
 #ifdef HAVE_SOLARIS_CDROM
 
-static const char _rcsid[] = "$Id: solaris.c,v 1.11 2006/03/17 19:06:51 rocky Exp $";
+static char ** cdio_get_devices_solaris_cXtYdZs2(int flag);
 
 #ifdef HAVE_GLOB_H
 #include <glob.h>
@@ -64,6 +61,7 @@ static const char _rcsid[] = "$Id: solaris.c,v 1.11 2006/03/17 19:06:51 rocky Ex
 #include <sys/stat.h>
 #include <sys/types.h>
 #include <sys/ioctl.h>
+#include <dirent.h>
 #include "cdtext_private.h"
 
 /* not defined in dkio.h yet */
@@ -74,7 +72,9 @@ static const char _rcsid[] = "$Id: solaris.c,v 1.11 2006/03/17 19:06:51 rocky Ex
 typedef  enum {
     _AM_NONE,
     _AM_SUN_CTRL_ATAPI,
-    _AM_SUN_CTRL_SCSI
+    _AM_SUN_CTRL_SCSI,
+    _AM_MMC_RDWR,
+    _AM_MMC_RDWR_EXCL
 #if FINISHED
     _AM_READ_CD,
     _AM_READ_10
@@ -98,7 +98,7 @@ typedef struct {
 } _img_private_t;
 
 static track_format_t get_track_format_solaris(void *p_user_data, 
-					       track_t i_track);
+                                               track_t i_track);
 
 static access_mode_t 
 str_to_access_mode_solaris(const char *psz_access_mode) 
@@ -111,9 +111,13 @@ str_to_access_mode_solaris(const char *psz_access_mode)
     return _AM_SUN_CTRL_SCSI; /* force ATAPI to be SCSI */
   else if (!strcmp(psz_access_mode, "SCSI"))
     return _AM_SUN_CTRL_SCSI;
+  else if (!strcmp(psz_access_mode, "MMC_RDWR"))
+    return _AM_MMC_RDWR;
+  else if (!strcmp(psz_access_mode, "MMC_RDWR_EXCL"))
+    return _AM_MMC_RDWR_EXCL;
   else {
     cdio_warn ("unknown access type: %s. Default SCSI used.", 
-	       psz_access_mode);
+               psz_access_mode);
     return default_access_mode;
   }
 }
@@ -139,7 +143,7 @@ audio_pause_solaris (void *p_user_data)
 */
 static driver_return_code_t
 audio_play_msf_solaris (void *p_user_data, msf_t *p_start_msf, 
-			msf_t *p_end_msf)
+                        msf_t *p_end_msf)
 {
 
   const _img_private_t *p_env = p_user_data;
@@ -163,7 +167,7 @@ audio_play_msf_solaris (void *p_user_data, msf_t *p_start_msf,
 */
 static driver_return_code_t
 audio_play_track_index_solaris (void *p_user_data, 
-				cdio_track_index_t *p_track_index)
+                                cdio_track_index_t *p_track_index)
 {
 
   const _img_private_t *p_env = p_user_data;
@@ -178,7 +182,7 @@ audio_play_track_index_solaris (void *p_user_data,
 */
 static driver_return_code_t
 audio_read_subchannel_solaris (void *p_user_data, 
-			       cdio_subchannel_t *p_subchannel)
+                               cdio_subchannel_t *p_subchannel)
 {
   const _img_private_t *p_env = p_user_data;
   struct cdrom_subchnl subchannel;
@@ -233,7 +237,7 @@ audio_resume_solaris (void *p_user_data)
 */
 static driver_return_code_t
 audio_set_volume_solaris (void *p_user_data, 
-			  cdio_audio_volume_t *p_volume) {
+                          cdio_audio_volume_t *p_volume) {
 
   const _img_private_t *p_env = p_user_data;
   return ioctl(p_env->gen.fd, CDROMVOLCTRL, p_volume);
@@ -252,17 +256,86 @@ audio_stop_solaris (void *p_user_data)
   return ioctl(p_env->gen.fd, CDROMSTOP);
 }
 
+static int
+cdio_decode_btl_number(char **cpt, int stopper, int *no)
+{
+  *no = 0;
+  for ((*cpt)++; **cpt != stopper; (*cpt)++) {
+    if (**cpt < '0' || **cpt > '9')
+      return 0;
+    *no = *no * 10 + **cpt - '0';
+  }
+  return 1;
+}
+
+/* Read bus, target, lun from name "cXtYdZs2".
+   Return 0 if name is not of the desired form.
+*/
+static int
+cdio_decode_btl_solaris(char *name, int *busno, int *tgtno, int *lunno,
+                        int flag)
+{
+  char *cpt;
+  int ret;
+
+  *busno = *tgtno = *lunno = -1;
+  cpt = name;
+  if (*cpt != 'c')
+    return 0;
+  ret = cdio_decode_btl_number(&cpt, 't', busno);
+  if (ret <= 0)
+    return ret;
+  ret = cdio_decode_btl_number(&cpt, 'd', tgtno);
+  if (ret <= 0)
+    return ret;
+  ret = cdio_decode_btl_number(&cpt, 's', lunno);
+  if (ret <= 0)
+    return ret;
+  cpt++;
+  if (*cpt != '2' || *(cpt + 1) != 0)
+    return 0;
+  return 1;
+}
+
+static int
+set_scsi_tuple_solaris (_img_private_t *p_env)
+{
+  int bus_no = -1, host_no = -1, channel_no = -1, target_no = -1, lun_no = -1;
+  int ret;
+  char tuple[160], *cpt;
+
+  cpt = strrchr(p_env->gen.source_name, '/');
+  if (cpt == NULL)
+    cpt = p_env->gen.source_name;
+  else
+    cpt++;
+  ret = cdio_decode_btl_solaris(cpt, &bus_no, &target_no, &lun_no, 0);
+  if (ret <= 0)
+    return(ret);
+  host_no = bus_no;
+  channel_no = 0;
+  snprintf(tuple, sizeof(tuple)-1, "%d,%d,%d,%d,%d",
+          bus_no, host_no, channel_no, target_no, lun_no);
+  p_env->gen.scsi_tuple = strdup(tuple);
+  return 1;
+}
+
 /*!
   Initialize CD device.
  */
 static bool
 init_solaris (_img_private_t *p_env)
 {
+  int open_flags = O_RDONLY | O_NDELAY;
 
-  if (!cdio_generic_init(p_env, O_RDONLY)) return false;
-  
-  p_env->access_mode = _AM_SUN_CTRL_SCSI;    
+  if (_AM_MMC_RDWR != p_env->access_mode &&
+      _AM_MMC_RDWR_EXCL != p_env->access_mode)
+    /* (was once set to _AM_SUN_CTRL_SCSI unconditionally) */
+    p_env->access_mode = _AM_SUN_CTRL_SCSI;
 
+  if (!cdio_generic_init(p_env, open_flags))
+    return false;
+  set_scsi_tuple_solaris(p_env);
   return true;
 }
 
@@ -272,33 +345,87 @@ init_solaris (_img_private_t *p_env)
   p_user_data   internal CD structure.
   i_timeout_ms   time in milliseconds we will wait for the command
                 to complete. 
-  i_cdb	        Size of p_cdb
-  p_cdb	        CDB bytes. 
-  e_direction	direction the transfer is to go.
-  i_buf	        Size of buffer
-  p_buf	        Buffer for data, both sending and receiving
+  i_cdb         Size of p_cdb
+  p_cdb         CDB bytes. 
+  e_direction   direction the transfer is to go.
+  i_buf         Size of buffer
+  p_buf         Buffer for data, both sending and receiving
  */
 static driver_return_code_t
-run_mmc_cmd_solaris( void *p_user_data, unsigned int i_timeout_ms,
-		     unsigned int i_cdb, const mmc_cdb_t *p_cdb, 
-		     cdio_mmc_direction_t e_direction, 
-		     unsigned int i_buf, /*in/out*/ void *p_buf )
+run_mmc_cmd_solaris(void *p_user_data, unsigned int i_timeout_ms,
+                    unsigned int i_cdb, const mmc_cdb_t *p_cdb, 
+                    cdio_mmc_direction_t e_direction, 
+                    unsigned int i_buf, /*in/out*/ void *p_buf)
 {
-  const _img_private_t *p_env = p_user_data;
+  _img_private_t *p_env = p_user_data;
   struct uscsi_cmd cgc;
+  int i_rc;
+  cdio_mmc_request_sense_t sense;
+  unsigned char *u_sense = (unsigned char *) &sense;
 
   memset (&cgc, 0, sizeof (struct uscsi_cmd));
   cgc.uscsi_cdb = (caddr_t) p_cdb;
 
-  cgc.uscsi_flags = SCSI_MMC_DATA_READ == e_direction ? 
-    USCSI_READ : USCSI_WRITE;
+  /* See: man uscsi
+          http://docs.sun.com/app/docs/doc/816-5177/uscsi-7i?a=view
+  */
+  p_env->gen.scsi_mmc_sense_valid = 0;
+  memset(u_sense, 0, sizeof(sense));
+  cgc.uscsi_rqbuf = (caddr_t) u_sense;
+  cgc.uscsi_rqlen = sizeof(sense); 
 
+  /* No error messages, no retries, do not execute with other commands, 
+     request sense data
+  */
+  cgc.uscsi_flags = USCSI_SILENT | USCSI_DIAGNOSE | USCSI_ISOLATE
+                    | USCSI_RQENABLE;
+
+  if (SCSI_MMC_DATA_READ == e_direction)
+    cgc.uscsi_flags |= USCSI_READ;
+  else if (SCSI_MMC_DATA_WRITE == e_direction)
+    cgc.uscsi_flags |= USCSI_WRITE;
   cgc.uscsi_timeout = msecs2secs(i_timeout_ms);
   cgc.uscsi_bufaddr = p_buf;   
   cgc.uscsi_buflen  = i_buf;
   cgc.uscsi_cdblen  = i_cdb;
   
-  return ioctl(p_env->gen.fd, USCSICMD, &cgc);
+  i_rc = ioctl(p_env->gen.fd, USCSICMD, &cgc);
+
+  /* Record SCSI sense reply for API call mmc_last_cmd_sense().
+  */
+  if (sense.additional_sense_len) { /* sense data available */
+    int sense_size = sense.additional_sense_len + 8;
+
+    if (sense_size > sizeof(sense))
+      sense_size = sizeof(sense);
+    memcpy((void *) p_env->gen.scsi_mmc_sense,  &sense, sense_size);
+    p_env->gen.scsi_mmc_sense_valid = sense_size;
+  } 
+
+  if (0 == i_rc) return DRIVER_OP_SUCCESS;
+  if (-1 == i_rc) {
+    cdio_info ("ioctl USCSICMD failed: %s", strerror(errno));  
+    switch (errno) {
+    case EPERM:
+      return DRIVER_OP_NOT_PERMITTED;
+      break;
+    case EINVAL:
+      return DRIVER_OP_BAD_PARAMETER;
+      break;
+    case EFAULT:
+      return DRIVER_OP_BAD_POINTER;
+      break;
+    case EIO: 
+    default:
+      return DRIVER_OP_ERROR;
+      break;
+    }
+  }
+  else if (i_rc < -1)
+    return DRIVER_OP_ERROR;
+  else
+    /*Not sure if this the best thing, but we'll use anyway. */
+    return DRIVER_OP_SUCCESS;
 }
 
 /*!
@@ -311,18 +438,14 @@ run_mmc_cmd_solaris( void *p_user_data, unsigned int i_timeout_ms,
 
 static int
 _read_audio_sectors_solaris (void *p_user_data, void *data, lsn_t i_lsn, 
-			     unsigned int i_blocks)
+                             unsigned int i_blocks)
 {
-  struct cdrom_msf solaris_msf;
   msf_t _msf;
   struct cdrom_cdda cdda;
 
   _img_private_t *p_env = p_user_data;
 
   cdio_lba_to_msf (cdio_lsn_to_lba(i_lsn), &_msf);
-  solaris_msf.cdmsf_min0   = cdio_from_bcd8(_msf.m);
-  solaris_msf.cdmsf_sec0   = cdio_from_bcd8(_msf.s);
-  solaris_msf.cdmsf_frame0 = cdio_from_bcd8(_msf.f);
   
   if (p_env->gen.ioctls_debugged == 75)
     cdio_debug ("only displaying every 75th ioctl from now on");
@@ -332,7 +455,7 @@ _read_audio_sectors_solaris (void *p_user_data, void *data, lsn_t i_lsn,
   
   if (p_env->gen.ioctls_debugged < 75 
       || (p_env->gen.ioctls_debugged < (30 * 75)  
-	  && p_env->gen.ioctls_debugged % 75 == 0)
+          && p_env->gen.ioctls_debugged % 75 == 0)
       || p_env->gen.ioctls_debugged % (30 * 75) == 0)
     cdio_debug ("reading %d", i_lsn);
   
@@ -340,7 +463,7 @@ _read_audio_sectors_solaris (void *p_user_data, void *data, lsn_t i_lsn,
 
   if (i_blocks > 60) {
     cdio_warn("%s:\n", 
-	      "we can't handle reading more than 60 blocks. Reset to 60");
+              "we can't handle reading more than 60 blocks. Reset to 60");
   }
 
   cdda.cdda_addr    = i_lsn;
@@ -363,7 +486,7 @@ _read_audio_sectors_solaris (void *p_user_data, void *data, lsn_t i_lsn,
  */
 static driver_return_code_t
 _read_mode1_sector_solaris (void *p_env, void *data, lsn_t i_lsn, 
-			    bool b_form2)
+                            bool b_form2)
 {
 
 #if FIXED
@@ -379,7 +502,7 @@ _read_mode1_sector_solaris (void *p_env, void *data, lsn_t i_lsn,
  */
 static driver_return_code_t
 _read_mode1_sectors_solaris (void *p_user_data, void *p_data, lsn_t i_lsn, 
-			     bool b_form2, unsigned int i_blocks)
+                             bool b_form2, unsigned int i_blocks)
 {
   _img_private_t *p_env = p_user_data;
   unsigned int i;
@@ -388,8 +511,8 @@ _read_mode1_sectors_solaris (void *p_user_data, void *p_data, lsn_t i_lsn,
 
   for (i = 0; i < i_blocks; i++) {
     if ( (retval = _read_mode1_sector_solaris (p_env, 
-					    ((char *)p_data) + (blocksize * i),
-					       i_lsn + i, b_form2)) )
+                                            ((char *)p_data) + (blocksize * i),
+                                               i_lsn + i, b_form2)) )
       return retval;
   }
   return DRIVER_OP_SUCCESS;
@@ -400,7 +523,7 @@ _read_mode1_sectors_solaris (void *p_user_data, void *p_data, lsn_t i_lsn,
  */
 static driver_return_code_t
 _read_mode2_sector_solaris (void *p_user_data, void *p_data, lsn_t i_lsn, 
-			    bool b_form2)
+                            bool b_form2)
 {
   char buf[CDIO_CD_FRAMESIZE_RAW] = { 0, };
   struct cdrom_msf solaris_msf;
@@ -423,11 +546,11 @@ _read_mode2_sector_solaris (void *p_user_data, void *p_data, lsn_t i_lsn,
   
   if (p_env->gen.ioctls_debugged < 75 
       || (p_env->gen.ioctls_debugged < (30 * 75)  
-	  && p_env->gen.ioctls_debugged % 75 == 0)
+          && p_env->gen.ioctls_debugged % 75 == 0)
       || p_env->gen.ioctls_debugged % (30 * 75) == 0)
     cdio_debug ("reading %2.2d:%2.2d:%2.2d",
-		solaris_msf.cdmsf_min0, solaris_msf.cdmsf_sec0, 
-		solaris_msf.cdmsf_frame0);
+                solaris_msf.cdmsf_min0, solaris_msf.cdmsf_sec0, 
+                solaris_msf.cdmsf_frame0);
   
   p_env->gen.ioctls_debugged++;
   
@@ -459,7 +582,7 @@ _read_mode2_sector_solaris (void *p_user_data, void *p_data, lsn_t i_lsn,
  */
 static driver_return_code_t
 _read_mode2_sectors_solaris (void *p_user_data, void *data, lsn_t i_lsn, 
-			     bool b_form2, unsigned int i_blocks)
+                             bool b_form2, unsigned int i_blocks)
 {
   _img_private_t *p_env = p_user_data;
   unsigned int i;
@@ -468,8 +591,8 @@ _read_mode2_sectors_solaris (void *p_user_data, void *data, lsn_t i_lsn,
 
   for (i = 0; i < i_blocks; i++) {
     if ( (retval = _read_mode2_sector_solaris (p_env, 
-					    ((char *)data) + (blocksize * i),
-					       i_lsn + i, b_form2)) )
+                                            ((char *)data) + (blocksize * i),
+                                               i_lsn + i, b_form2)) )
       return retval;
   }
   return 0;
@@ -572,9 +695,9 @@ read_toc_solaris (void *p_user_data)
   p_env->tocent[p_env->tochdr.cdth_trk1].cdte_format = CDIO_CDROM_MSF;
 
   if (ioctl(p_env->gen.fd, CDROMREADTOCENTRY, 
-	    &p_env->tocent[p_env->tochdr.cdth_trk1]) == -1 ) {
+            &p_env->tocent[p_env->tochdr.cdth_trk1]) == -1 ) {
     cdio_warn("%s: %s\n", 
-	     "error in ioctl CDROMREADTOCENTRY for lead-out",
+             "error in ioctl CDROMREADTOCENTRY for lead-out",
             strerror(errno));
     return false;
   }
@@ -607,6 +730,13 @@ eject_media_solaris (void *p_user_data) {
   return DRIVER_OP_ERROR;
 }
 
+static bool
+is_mmc_supported(void *user_data)
+{
+    _img_private_t *env = user_data;
+    return (_AM_NONE == env->access_mode) ? false : true;
+}
+
 /*!
   Return the value associated with the key "arg".
 */
@@ -623,9 +753,17 @@ get_arg_solaris (void *p_user_data, const char key[])
       return "ATAPI";
     case _AM_SUN_CTRL_SCSI:
       return "SCSI";
+    case _AM_MMC_RDWR:
+      return "MMC_RDWR";
+    case _AM_MMC_RDWR_EXCL:
+      return "MMC_RDWR_EXCL";
     case _AM_NONE:
       return "no access method";
     }
+  } else if (!strcmp (key, "scsi-tuple")) {
+    return p_env->gen.scsi_tuple;
+  } else if (!strcmp (key, "mmc-supported?")) {
+    return is_mmc_supported(p_user_data) ? "true" : "false";
   } 
   return NULL;
 }
@@ -650,6 +788,29 @@ get_blocksize_solaris (void *p_user_data) {
   }
 }
 
+#ifdef HAVE_SOLARIS_CDROM
+/*!
+  Return a string containing the default CD device if none is specified.
+  This call does not assume a fixed default drive address but rather uses
+  the first drive that gets enumerated by cdio_get_devices_solaris_cXtYdZs2().
+ */
+static char *
+cdio_get_default_cXtYdZs2(void)
+{
+  char **devlist, *result = NULL;
+
+  devlist = cdio_get_devices_solaris_cXtYdZs2(1);
+  if(devlist != NULL) {
+    if(devlist[0] != NULL)
+      result = strdup(devlist[0]);
+    free(devlist);
+  }
+  if(result != NULL)
+    return result;
+  return strdup(DEFAULT_CDIO_DEVICE);
+}
+#endif
+
 /*!
   Return a string containing the default CD device if none is specified.
  */
@@ -662,16 +823,23 @@ cdio_get_default_device_solaris(void)
   char *device;
   struct stat stb;
 
+  /* vold and its directory /vol have been replaced by "Tamarack" which
+     is based on hald. This happened in 2006.
+  */
+  if(stat("/vol", &stb) == -1)
+    return cdio_get_default_cXtYdZs2();
+  if((stb.st_mode & S_IFMT) != S_IFDIR)
+    return cdio_get_default_cXtYdZs2();
+
   if ((volume_device = getenv("VOLUME_DEVICE")) != NULL &&
       (volume_name   = getenv("VOLUME_NAME"))   != NULL &&
       (volume_action = getenv("VOLUME_ACTION")) != NULL &&
       strcmp(volume_action, "insert") == 0) {
-
-    device = calloc(1, strlen(volume_device) 
-				  + strlen(volume_name) + 2);
+    uint len = strlen(volume_device) + strlen(volume_name) + 2;
+    device = calloc(1, len);
     if (device == NULL)
       return strdup(DEFAULT_CDIO_DEVICE);
-    sprintf(device, "%s/%s", volume_device, volume_name);
+    snprintf(device, len, "%s/%s", volume_device, volume_name);
     if (stat(device, &stb) != 0 || !S_ISCHR(stb.st_mode)) {
       free(device);
       return strdup(DEFAULT_CDIO_DEVICE);
@@ -680,8 +848,9 @@ cdio_get_default_device_solaris(void)
   }
   /* Check if it could be a Solaris media*/
   if((stat(DEFAULT_CDIO_DEVICE, &stb) == 0) && S_ISDIR(stb.st_mode)) {
-    device = calloc(1, strlen(DEFAULT_CDIO_DEVICE) + 4);
-    sprintf(device, "%s/s0", DEFAULT_CDIO_DEVICE);
+    uint len = strlen(DEFAULT_CDIO_DEVICE + 4);
+    device = calloc(1, len);
+    snprintf(device, len, "%s/s0", DEFAULT_CDIO_DEVICE);
     return device;
   }
   return strdup(DEFAULT_CDIO_DEVICE);
@@ -703,7 +872,7 @@ get_discmode_solaris (void *p_user_data)
   /* Get the media info */
   if((ret = ioctl(p_env->gen.fd, DKIOCGMEDIAINFO, &media)) != 0) {
      cdio_warn ("DKIOCGMEDIAINFO failed: %s\n", strerror(errno));
-	 return CDIO_DISC_MODE_NO_INFO;
+         return CDIO_DISC_MODE_NO_INFO;
   }
   switch(media.dki_media_type) {
   case DK_CDROM:
@@ -711,13 +880,13 @@ get_discmode_solaris (void *p_user_data)
   case DK_CDRW:
   /* Do cdrom detection */
   break;
-  case DK_DVDROM:	return CDIO_DISC_MODE_DVD_ROM;
-  case DK_DVDR:		discmode = CDIO_DISC_MODE_DVD_R;
+  case DK_DVDROM:       return CDIO_DISC_MODE_DVD_ROM;
+  case DK_DVDR:         discmode = CDIO_DISC_MODE_DVD_R;
   break;
-  case DK_DVDRAM:	discmode = CDIO_DISC_MODE_DVD_RAM;
+  case DK_DVDRAM:       discmode = CDIO_DISC_MODE_DVD_RAM;
   break;
   case DK_DVDRW:
-  case DK_DVDRW+1:	discmode = CDIO_DISC_MODE_DVD_RW;
+  case DK_DVDRW+1:      discmode = CDIO_DISC_MODE_DVD_RW;
   break;
   default: /* no valid match */
   return CDIO_DISC_MODE_NO_INFO; 
@@ -756,44 +925,44 @@ get_discmode_solaris (void *p_user_data)
     switch(track_fmt) {
     case TRACK_FORMAT_AUDIO:
       switch(discmode) {
-	case CDIO_DISC_MODE_NO_INFO:
-	  discmode = CDIO_DISC_MODE_CD_DA;
-	  break;
-	case CDIO_DISC_MODE_CD_DA:
-	case CDIO_DISC_MODE_CD_MIXED: 
-	case CDIO_DISC_MODE_ERROR: 
-	  /* No change*/
-	  break;
+        case CDIO_DISC_MODE_NO_INFO:
+          discmode = CDIO_DISC_MODE_CD_DA;
+          break;
+        case CDIO_DISC_MODE_CD_DA:
+        case CDIO_DISC_MODE_CD_MIXED: 
+        case CDIO_DISC_MODE_ERROR: 
+          /* No change*/
+          break;
       default:
-	  discmode = CDIO_DISC_MODE_CD_MIXED;
+          discmode = CDIO_DISC_MODE_CD_MIXED;
       }
       break;
     case TRACK_FORMAT_XA:
       switch(discmode) {
-	case CDIO_DISC_MODE_NO_INFO:
-	  discmode = CDIO_DISC_MODE_CD_XA;
-	  break;
-	case CDIO_DISC_MODE_CD_XA:
-	case CDIO_DISC_MODE_CD_MIXED: 
-	case CDIO_DISC_MODE_ERROR: 
-	  /* No change*/
-	  break;
+        case CDIO_DISC_MODE_NO_INFO:
+          discmode = CDIO_DISC_MODE_CD_XA;
+          break;
+        case CDIO_DISC_MODE_CD_XA:
+        case CDIO_DISC_MODE_CD_MIXED: 
+        case CDIO_DISC_MODE_ERROR: 
+          /* No change*/
+          break;
       default:
-	discmode = CDIO_DISC_MODE_CD_MIXED;
+        discmode = CDIO_DISC_MODE_CD_MIXED;
       }
       break;
     case TRACK_FORMAT_DATA:
       switch(discmode) {
-	case CDIO_DISC_MODE_NO_INFO:
-	  discmode = CDIO_DISC_MODE_CD_DATA;
-	  break;
-	case CDIO_DISC_MODE_CD_DATA:
-	case CDIO_DISC_MODE_CD_MIXED: 
-	case CDIO_DISC_MODE_ERROR: 
-	  /* No change*/
-	  break;
+        case CDIO_DISC_MODE_NO_INFO:
+          discmode = CDIO_DISC_MODE_CD_DATA;
+          break;
+        case CDIO_DISC_MODE_CD_DATA:
+        case CDIO_DISC_MODE_CD_MIXED: 
+        case CDIO_DISC_MODE_ERROR: 
+          /* No change*/
+          break;
       default:
-	discmode = CDIO_DISC_MODE_CD_MIXED;
+        discmode = CDIO_DISC_MODE_CD_MIXED;
       }
       break;
     case TRACK_FORMAT_ERROR:
@@ -812,7 +981,7 @@ get_discmode_solaris (void *p_user_data)
 */
 static driver_return_code_t 
 get_last_session_solaris (void *p_user_data, 
-			  /*out*/ lsn_t *i_last_session_lsn)
+                          /*out*/ lsn_t *i_last_session_lsn)
 {
   const _img_private_t *p_env = p_user_data;
   int i_rc;
@@ -991,6 +1160,115 @@ close_tray_solaris (const char *psz_device)
 #endif /*HAVE_SOLARIS_CDROM*/
 }
 
+#ifdef HAVE_SOLARIS_CDROM
+/*!
+  Return an array of strings giving possible CD devices.
+  New method after demise of vold in 2006. 
+ */
+/* flag bit0= need only the first drive
+*/
+static char **
+cdio_get_devices_solaris_cXtYdZs2(int flag)
+{
+  int busno, tgtno, lunno, ret;
+  char volpath[160];
+  char **drives = NULL;
+  unsigned int i_files=0;
+  DIR *dir = NULL;
+  struct dirent *entry;
+
+#ifdef LIBCDIO_SOLARIS_WITH_CD_INQUIRY
+  CdIo_t *cdio = NULL;
+  mmc_cdb_t cdb = {{0, }};
+  int timeout_ms;
+  driver_return_code_t i_status;
+  char reply[36];
+  static unsigned char spc_inquiry[] = { 0x12, 0, 0, 0, 36, 0 };
+#else
+  struct dk_cinfo cinfo;
+  int fd = -1;
+#endif
+
+  static int recursion = 0;
+
+  if (recursion) {
+    fprintf(stderr, "Program error ! Recursion of cdio_get_devices_solaris_cXtYdZs2()\n");
+    return NULL;
+  }
+  recursion = 1;
+
+  dir = opendir("/dev/rdsk");
+  if (dir == NULL) {
+    cdio_warn ("opendir(\"/dev/rdsk\") failed: %s\n", strerror(errno));  
+    goto ex;
+  }
+  while (1) {
+    entry = readdir(dir);
+    if (entry == NULL) {
+      if (errno) {
+        cdio_warn ("readdir(/dev/rdsk) failed: %s\n", strerror(errno));  
+        goto ex;
+      }
+  break;
+    }
+    ret = cdio_decode_btl_solaris(entry->d_name, &busno, &tgtno, &lunno, 0);
+    if (ret < 0)
+      goto ex;
+    if (ret == 0)
+  continue; /* not cXtYdZs2 */
+ 
+    if (strlen(entry->d_name) > sizeof(volpath) - 11)
+  continue;
+    snprintf(volpath, sizeof(volpath), "/dev/rdsk/%s", entry->d_name);
+
+#ifdef LIBCDIO_SOLARIS_WITH_CD_INQUIRY
+
+    cdio = cdio_open_am_solaris(volpath, "MMC_RDWR");
+    if(cdio == NULL)
+  continue;
+    memcpy(cdb.field, spc_inquiry, 6);
+    timeout_ms = 10000;
+    i_status = run_mmc_cmd_solaris(cdio->env, timeout_ms,
+                                   6, &cdb, SCSI_MMC_DATA_READ,
+                                   (unsigned int) spc_inquiry[4], reply);
+    cdio_destroy(cdio);
+    cdio = NULL;
+    if (i_status != 0)
+  continue;
+
+    /* SBC-3 , table 83 , PERIPHERAL DEVICE TYPE : 5 = CD/DVD device */
+    if((reply[0] & 0x1F) != 5)
+  continue;
+
+#else /* LIBCDIO_SOLARIS_WITH_CD_INQUIRY */
+
+    fd = open(volpath, O_RDONLY | O_NDELAY);
+    if (fd < 0)
+  continue;
+    /* See man dkio */
+    ret = ioctl(fd, DKIOCINFO, &cinfo);
+    close(fd);
+    fd = -1;
+    if (ret < 0)
+  continue;
+    if (cinfo.dki_ctype != DKC_CDROM)
+  continue;
+
+#endif /* ! LIBCDIO_SOLARIS_WITH_CD_INQUIRY */
+
+    cdio_add_device_list(&drives, volpath, &i_files);
+    if(flag & 1)
+      goto ex; /* Only the first drive is desired */
+  }
+ex:;
+  recursion = 0;
+  if(dir != NULL)
+    closedir(dir);
+  cdio_add_device_list(&drives, NULL, &i_files);
+  return drives;
+}
+#endif /*HAVE_SOLARIS_CDROM*/
+
 /*!
   Return an array of strings giving possible CD devices.
  */
@@ -1007,6 +1285,17 @@ cdio_get_devices_solaris (void)
 #ifdef HAVE_GLOB_H
   unsigned int i;
   glob_t globbuf;
+#endif
+
+  /* vold and its directory /vol have been replaced by "Tamarack" which
+     is based on hald. This happened in 2006.
+  */
+  if(stat("/vol", &st) == -1)
+    return cdio_get_devices_solaris_cXtYdZs2(0);
+  if((st.st_mode & S_IFMT) != S_IFDIR)
+    return cdio_get_devices_solaris_cXtYdZs2(0);
+
+#ifdef HAVE_GLOB_H
 
   globbuf.gl_offs = 0;
   glob("/vol/dev/aliases/cdrom*", GLOB_DOOFFS, NULL, &globbuf);
@@ -1016,10 +1305,10 @@ cdio_get_devices_solaris (void)
 
     /* Check if this is a directory, if so it's probably Solaris media */
     if(S_ISDIR(st.st_mode)) {
-      sprintf(volpath, "%s/s0", globbuf.gl_pathv[i]);
+      snprintf(volpath, sizeof(volpath), "%s/s0", globbuf.gl_pathv[i]);
       if(stat(volpath, &st) == 0)
         cdio_add_device_list(&drives, volpath, &i_files);
-	}else
+        }else
       cdio_add_device_list(&drives, globbuf.gl_pathv[i], &i_files);
   }
   globfree(&globbuf);
@@ -1027,7 +1316,7 @@ cdio_get_devices_solaris (void)
   if(stat(DEFAULT_CDIO_DEVICE, &st) == 0) {
     /* Check if this is a directory, if so it's probably Solaris media */
     if(S_ISDIR(st.st_mode)) {
-      sprintf(volpath, "%s/s0", DEFAULT_CDIO_DEVICE);
+      snprintf(volpath, sizeof(volpath), "%s/s0", DEFAULT_CDIO_DEVICE);
       if(stat(volpath, &st) == 0)
         cdio_add_device_list(&drives, volpath, &i_files);
     }else
@@ -1084,6 +1373,7 @@ cdio_open_am_solaris (const char *psz_orig_source, const char *access_mode)
   _funcs.get_blocksize          = get_blocksize_solaris;
 #endif
   _funcs.get_cdtext             = get_cdtext_generic;
+  _funcs.get_cdtext_raw         = read_cdtext_generic;
   _funcs.get_default_device     = cdio_get_default_device_solaris;
   _funcs.get_devices            = cdio_get_devices_solaris;
   _funcs.get_disc_last_lsn      = get_disc_last_lsn_solaris;
@@ -1122,16 +1412,19 @@ cdio_open_am_solaris (const char *psz_orig_source, const char *access_mode)
 
   _data                         = calloc(1, sizeof (_img_private_t));
 
-  _data->access_mode    = _AM_SUN_CTRL_SCSI;
+  _data->access_mode    = str_to_access_mode_solaris(access_mode);
   _data->gen.init       = false;
   _data->gen.fd         = -1;
   _data->gen.toc_init   = false;
-  _data->gen.b_cdtext_init  = false;
   _data->gen.b_cdtext_error = false;
 
   if (NULL == psz_orig_source) {
     psz_source = cdio_get_default_device_solaris();
-    if (NULL == psz_source) return NULL;
+    if (NULL == psz_source) {
+      free(_data);
+      return NULL;
+    }
+
     _set_arg_solaris(_data, "source", psz_source);
     free(psz_source);
   } else {
@@ -1156,6 +1449,7 @@ cdio_open_am_solaris (const char *psz_orig_source, const char *access_mode)
     return ret;
   else {
     cdio_generic_free (_data);
+    free(ret);
     return NULL;
   }
 
