@@ -1,6 +1,6 @@
 /* -*- C -*-
   Copyright (C) 2009 Thomas Schmitt <scdbackup@gmx.net>
-  Copyright (C) 2010 Rocky Bernstein <rocky@gnu.org>
+  Copyright (C) 2010, 2012-2013 Rocky Bernstein <rocky@gnu.org>
 
   This program is free software: you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
@@ -16,16 +16,13 @@
   along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */
 
-/* 
+/*
    Regression test for MMC commands involving read/write access.
 */
 #ifdef HAVE_CONFIG_H
-# include "config.h"
-# define __CDIO_CONFIG_H__ 1
+#include "config.h"
+#define __CDIO_CONFIG_H__ 1
 #endif
-#include <cdio/cdio.h>
-#include <cdio/logging.h>
-#include <cdio/mmc_cmds.h>
 
 #ifdef HAVE_STDIO_H
 #include <stdio.h>
@@ -36,23 +33,42 @@
 #ifdef HAVE_STDLIB_H
 #include <stdlib.h>
 #endif
+#ifdef HAVE_UNISTD_H
+#include <unistd.h>
+#endif
+#ifdef HAVE_STRING_H
 #include <string.h>
+#endif
+#if !defined(HAVE_SLEEP)
+#  if defined(HAVE_USLEEP)
+#     define sleep(s) usleep(1000000*s)
+#  elif defined(_WIN32)
+#     include <windows.h>
+#     define sleep(s) Sleep(1000*s)
+#  else
+#     define sleep(s) { int i; for(i=0; i<=1000*s; i++); }
+#  endif
+#endif
+
+#include <cdio/cdio.h>
+#include <cdio/logging.h>
+#include <cdio/mmc_cmds.h>
 
 #define SKIP_TEST 77
 
 /* gcc may warn if no prototypes are given before function definition */
 
 static int handle_outcome(CdIo_t *p_cdio, int i_status,
-			       unsigned int *pi_sense_avail, 
-			       cdio_mmc_request_sense_t * p_sense_reply, 
+			       unsigned int *pi_sense_avail,
+			       cdio_mmc_request_sense_t * p_sense_reply,
 			       unsigned int i_flag);
 
 static int load_eject(CdIo_t *p_cdio, unsigned int *pi_sense_avail,
-		      cdio_mmc_request_sense_t * p_sense_reply, 
+		      cdio_mmc_request_sense_t * p_sense_reply,
 		      unsigned int i_flag);
 
 static void print_status_sense(int i_status, int sense_valid,
-				    cdio_mmc_request_sense_t *, 
+				    cdio_mmc_request_sense_t *,
 				    unsigned int i_flag);
 
 static int test_eject_load_cycle(CdIo_t *p_cdio, unsigned int i_flag);
@@ -62,19 +78,19 @@ static int test_eject_test_load(CdIo_t *p_cdio, unsigned int i_flag);
 static int test_mode_select(CdIo_t *p_cdio,
 			    unsigned int *pi_sense_avail,
 			    cdio_mmc_request_sense_t *p_sense_reply,
-			    unsigned char *p_buf, unsigned int i_size, 
+			    unsigned char *p_buf, unsigned int i_size,
 			    unsigned int i_flag);
 
 static int mode_sense(CdIo_t *p_cdio, unsigned int *pi_sense_avail,
-		      cdio_mmc_request_sense_t * p_sense_reply, 
-		      unsigned int i_page_code, 
+		      cdio_mmc_request_sense_t * p_sense_reply,
+		      unsigned int i_page_code,
 		      unsigned int subpage_code, int i_alloc_len,
-		      unsigned char *buf, int *i_size, 
+		      unsigned char *buf, int *i_size,
 		      unsigned int i_flag);
 
-static int test_unit_ready(CdIo_t *p_cdio, 
+static int test_unit_ready(CdIo_t *p_cdio,
 			   unsigned int *pi_sense_avail,
-			   cdio_mmc_request_sense_t * p_sense_reply, 
+			   cdio_mmc_request_sense_t * p_sense_reply,
 			   unsigned int i_flag);
 
 static int test_rwr_mode_page(CdIo_t *p_cdio, unsigned int i_flag);
@@ -90,8 +106,8 @@ static int wait_for_drive(CdIo_t *p_cdio, unsigned int max_tries, unsigned int i
 */
 static int
 handle_outcome(CdIo_t *p_cdio, driver_return_code_t i_status,
-                    unsigned int *pi_sense_avail, 
-		    cdio_mmc_request_sense_t * p_sense_reply, 
+                    unsigned int *pi_sense_avail,
+		    cdio_mmc_request_sense_t * p_sense_reply,
 		    unsigned int i_flag)
 {
     cdio_mmc_request_sense_t *p_temp_sense_reply = NULL;
@@ -110,7 +126,7 @@ handle_outcome(CdIo_t *p_cdio, driver_return_code_t i_status,
 */
 static void
 print_status_sense(int i_status, int sense_valid,
-                        cdio_mmc_request_sense_t *p_sense_reply, 
+                        cdio_mmc_request_sense_t *p_sense_reply,
 			unsigned int i_flag)
 {
     if (!(i_flag & 1))
@@ -120,9 +136,9 @@ print_status_sense(int i_status, int sense_valid,
 	printf(":  KEY=%s (%1.1X), ASC= %2.2X , ASCQ= %2.2X",
 		mmc_sense_key2str[p_sense_reply->sense_key],
 		p_sense_reply->sense_key,
-		p_sense_reply->asc, 
+		p_sense_reply->asc,
 		p_sense_reply->ascq);
-    printf("\n");  
+    printf("\n");
 }
 
 
@@ -183,14 +199,14 @@ mode_sense(CdIo_t *p_cdio, unsigned int *pi_sense_avail,
 	   unsigned char *p_buf, int *pi_size, unsigned int i_flag)
 {
   driver_return_code_t i_status;
-  
+
   if (i_alloc_len < 10)
     return DRIVER_OP_BAD_PARAMETER;
 
   if (i_flag & 1)
     printf("mode_sense(0x%X, %X, %d) ... ",
 	   i_page_code, subpage_code, i_alloc_len);
-  
+
   i_status = mmc_mode_sense_10(p_cdio, p_buf, i_alloc_len, i_page_code);
   handle_outcome(p_cdio, i_status, pi_sense_avail, p_sense_reply, i_flag & 1);
   if (DRIVER_OP_SUCCESS != i_status)
@@ -219,7 +235,7 @@ mode_sense(CdIo_t *p_cdio, unsigned int *pi_sense_avail,
 */
 static int
 test_mode_select(CdIo_t *p_cdio,
-                 unsigned int *pi_sense_avail, 
+                 unsigned int *pi_sense_avail,
 		 cdio_mmc_request_sense_t *p_sense_reply,
                  unsigned char *p_buf, unsigned int i_size, unsigned int i_flag)
 {
@@ -227,9 +243,9 @@ test_mode_select(CdIo_t *p_cdio,
 
     if (i_size < 10)
 	return DRIVER_OP_BAD_PARAMETER;
-    
+
     if (i_flag & 1) {
-	printf("test_mode_select to drive: %d bytes\n", i_size);
+	printf("-- test_mode_select to drive: %d bytes\n", i_size);
 	for (i = 0; i < i_size; i++) {
 	    printf("%2.2X ", (unsigned int) p_buf[i]);
 	    if ((i % 20) == 19)
@@ -238,9 +254,9 @@ test_mode_select(CdIo_t *p_cdio,
 	if ((i % 20))
 	    printf("\n");
     }
-    
+
     if (i_flag & 1)
-	printf("test_mode_select(0x%X, %d, %d) ... ",
+	printf("-- test_mode_select(0x%X, %d, %d) ... ",
 	       (unsigned int) p_buf[8], (unsigned int) p_buf[9], i_size);
     i_status = mmc_mode_select_10(p_cdio, p_buf, i_size, 0x10, 10000);
     return handle_outcome(p_cdio, i_status, pi_sense_avail, p_sense_reply,
@@ -257,14 +273,14 @@ test_mode_select(CdIo_t *p_cdio,
  */
 static int
 test_unit_ready(CdIo_t *p_cdio,
-		unsigned int *pi_sense_avail, 
+		unsigned int *pi_sense_avail,
 		cdio_mmc_request_sense_t *p_sense_reply,
 		unsigned int i_flag)
 {
   int i_status;
 
   if (i_flag & 1)
-    printf("test_unit_ready ... ");
+    printf("-- test_unit_ready ... ");
   i_status = mmc_test_unit_ready(p_cdio, 0);
 
   return handle_outcome(p_cdio, i_status, pi_sense_avail, p_sense_reply,
@@ -288,7 +304,7 @@ wait_for_drive(CdIo_t *p_cdio, unsigned int i_max_tries, unsigned int i_flag)
     int i_ret, i;
     unsigned int i_sense_avail;
     cdio_mmc_request_sense_t sense_reply;
-  
+
     for (i = 0; i < i_max_tries; i++) {
 	i_ret = test_unit_ready(p_cdio, &i_sense_avail, &sense_reply, !!(i_flag & 1));
 	if (i_ret == 0) /* Unit is ready */
@@ -296,32 +312,32 @@ wait_for_drive(CdIo_t *p_cdio, unsigned int i_max_tries, unsigned int i_flag)
 	if (i_sense_avail < 18)
 	    return -1;
 	if (2 == sense_reply.sense_key && 0x04 == sense_reply.asc) {
-	    
+
 	    /* Not ready */;
-	    
+
 	} else if (6 == sense_reply.sense_key && 0x28 == sense_reply.asc &&
 		   0 == sense_reply.ascq) {
-	    
+
 	    /* Media change notice = try again */;
-	    
+
 	} else if (2 == sense_reply.sense_key && 0x3a == sense_reply.asc) {
-	    
+
 	    /* Medium not present */;
-	    
+
 	    if (!(i_flag & 2))
 		return 1;
 	} else if (0 == sense_reply.sense_key && 0 == sense_reply.asc) {
-	    
+
 	    /* Error with no sense */;
-	    
+
 	    return -1;
 	    break;
 	} else {
-	    
+
 	    /* Other error */;
-	    
+
 	    return 0;
-	} 
+	}
 	sleep(1);
     }
     fprintf(stderr, "wait_for_drive: Drive not ready after %d retries\n",
@@ -348,15 +364,15 @@ test_eject_load_cycle(CdIo_t *p_cdio, unsigned int i_flag)
     printf("test_eject_load_cycle: WARNING: EJECTING THE TRAY !\n");
     sleep(2);
     load_eject(p_cdio, &i_sense_avail, &sense_reply, 0 | (i_flag & 1));
-    
+
     printf("test_eject_load_cycle: waiting for 5 seconds. DO NOT TOUCH THE TRAY !\n");
     sleep(3);
-    
+
     /* Load asynchronously */
     printf("test_eject_load_cycle: WARNING: LOADING THE TRAY !\n");
     sleep(2);
     load_eject(p_cdio, &i_sense_avail, &sense_reply, 4 | 2 | (i_flag & 1));
-    
+
     /* Wait for drive attention */
     i_ret = wait_for_drive(p_cdio, 30, i_flag & 3);
     return i_ret;
@@ -434,7 +450,8 @@ test_rwr_mode_page(CdIo_t *p_cdio, unsigned int i_flag)
 			    page_code, subpage_code, i_alloc_len,
 			    buf, &i_size, 2 | (i_flag & 1));
     if (i_ret != 0) {
-	fprintf(stderr, "test_rwr_mode_page: Cannot obtain mode page 05h.\n");
+	fprintf(stderr,
+		"-- test_rwr_mode_page: Cannot obtain mode page 05h.\n");
 	return 0;
     }
     i_alloc_len = (i_size <= sizeof(buf)) ? i_size : sizeof(buf);
@@ -442,21 +459,22 @@ test_rwr_mode_page(CdIo_t *p_cdio, unsigned int i_flag)
 			    page_code, subpage_code, i_alloc_len,
 			    buf, &i_size, (i_flag & 1));
     if (i_ret != 0) {
-	fprintf(stderr, "test_rwr_mode_page: Cannot obtain mode page 05h.\n");
+	fprintf(stderr,
+		"-- test_rwr_mode_page: Cannot obtain mode page 05h.\n");
 	return 0;
     }
     memcpy(old_buf, buf, sizeof(buf));
     old_i_size = i_size;
-    
-    write_type = buf[10] & 0x0f; 
+
+    write_type = buf[10] & 0x0f;
     if (i_flag & 1)
 	printf("test_rwr_mode_page: Write type = %d (%s)\n",
 	       write_type, write_type < 4 ? w_types[write_type] : "Reserved");
-    
+
     /* Choose a conservative CD writer setting.
      */
     memset(buf, 0, 8);
-    if (write_type == 1) { /* is TAO -> make SAO */ 
+    if (write_type == 1) { /* is TAO -> make SAO */
 	buf[10] = (buf[10] & 0xf0) | 2;   /* SAO */
 	new_write_type = 2;
     } else {
@@ -467,11 +485,12 @@ test_rwr_mode_page(CdIo_t *p_cdio, unsigned int i_flag)
 					   variable Packet size */
     buf[12] = 8;                        /* Data Block Type : CD-ROM Mode 1 */
     buf[16] = 0;                        /* Session Format : "CD-DA or CD-ROM" */
-    
+
     i_ret = test_mode_select(p_cdio, &i_sense_avail, &sense_reply,
 			     buf, i_size, i_flag & 1);
     if (i_ret != 0) {
-	fprintf(stderr, "test_rwr_mode_page: Cannot set mode page 05h.\n");
+	fprintf(stderr,
+		"-- test_rwr_mode_page: Cannot set mode page 05h.\n");
 	if (DRIVER_OP_NOT_PERMITTED == i_ret) {
 	    fprintf(stderr,
 		    "test_rwr_mode_page: DRIVER_OP_NOT_PERMITTED with MODE SELECT.\n");
@@ -479,7 +498,7 @@ test_rwr_mode_page(CdIo_t *p_cdio, unsigned int i_flag)
 	}
 	return 0;
     }
-    
+
     /* Read mode page and check whether effect visible in buf[10] */
     i_ret = mode_sense(p_cdio, &i_sense_avail, &sense_reply,
 			    page_code, subpage_code, i_alloc_len,
@@ -489,12 +508,12 @@ test_rwr_mode_page(CdIo_t *p_cdio, unsigned int i_flag)
 	final_return = final_return > 0 ? 0 : final_return;
     } else if ((buf[10] & 0xf) != new_write_type) {
 	fprintf(stderr,
-		"test_rwr_mode_page: Mode page did not get into effect. (due %d , is %d)\n",
+		"test_rwr_mode_page: Mode page did not get into effect. (expected %d, got %d)\n",
 		new_write_type, buf[10] & 0xf);
 	/* One of my DVD burners does this if no media is loaded */
 	final_return = final_return > 0 ? 0 : final_return;
-    } 
-    
+    }
+
     /* Restore old mode page */
     i_ret = test_mode_select(p_cdio, &i_sense_avail, &sense_reply,
 			     old_buf, old_i_size, i_flag & 1);
@@ -507,7 +526,7 @@ test_rwr_mode_page(CdIo_t *p_cdio, unsigned int i_flag)
 	}
 	final_return = final_return > 0 ? 0 : final_return;
     }
-    
+
     /* Read mode page and check whether old_buf is in effect again */
     i_ret = mode_sense(p_cdio, &i_sense_avail, &sense_reply,
 		       page_code, subpage_code, i_alloc_len,
@@ -518,8 +537,8 @@ test_rwr_mode_page(CdIo_t *p_cdio, unsigned int i_flag)
     } else if (memcmp(buf, old_buf, i_size) != 0) {
 	fprintf(stderr,
 		"test_rwr_mode_page: Mode page was not restored to old state.\n");
-	final_return = -1;
-    } 
+	final_return = final_return > 0 ? -1 : final_return;
+    }
     if (i_flag & 1)
 	printf("test_rwr_mode_page: Mode page 2Ah restored to previous state\n");
     return final_return;
@@ -560,7 +579,7 @@ test_write(char *psz_drive_path, unsigned int i_flag)
 
     /* If true, then the tray will get ejected and loaded again */
     static bool with_tray_dance = false;
-    
+
     /* If true, then an asynchronous test loop will wait 30 s for loaded media */
     static bool test_cycle_with_media = false;
 
@@ -569,11 +588,11 @@ test_write(char *psz_drive_path, unsigned int i_flag)
     static bool emul_lack_of_wperm = false;
 
     p_cdio = cdio_open_am(psz_drive_path, DRIVER_DEVICE, "MMC_RDWR_EXCL");
-    if (!p_cdio) 
+    if (!p_cdio)
 	i_ret = SKIP_TEST - 16;
     else  {
 	const char *psz_access_mode = cdio_get_arg(p_cdio, "access-mode");
-	
+
 	if (0 != strncmp(psz_access_mode,
 			 "MMC_RDWR_EXCL", strlen("MMC_RDWR_EXCL"))) {
 	    fprintf(stderr,
@@ -586,7 +605,7 @@ test_write(char *psz_drive_path, unsigned int i_flag)
        reported by mmc_run_cmd() as INFO messages.
 	*/
 	cdio_loglevel_default = CDIO_LOG_WARN;
-	
+
 	/* Test availability of sense reply in case of unready drive.
 	   E.g. if the tray is already ejected.
 	*/
@@ -597,14 +616,14 @@ test_write(char *psz_drive_path, unsigned int i_flag)
 		    i_sense_avail);
 	    i_ret = 2; goto ex;
 	}
-	
+
 	if (emul_lack_of_wperm) { /* To check behavior with lack of w-permission */
-	    printf("test_write: SIMULATING LACK OF WRITE CAPABILITIES by access mode IOCTL\n");
+	    printf("-- test_write: SIMULATING LACK OF WRITE CAPABILITIES by access mode IOCTL\n");
 	    cdio_destroy(p_cdio);
 	    p_cdio = cdio_open_am(psz_drive_path, DRIVER_DEVICE, "IOCTL");
 	}
-	
-	
+
+
 	/* Test write permission */ /* Try whether a mode page 2Ah can be set */
 	i_ret = test_rwr_mode_page(p_cdio, b_verbose);
 	if (i_ret <= 0) {
@@ -612,10 +631,10 @@ test_write(char *psz_drive_path, unsigned int i_flag)
 		fprintf(stderr, "Error: test_rwr_mode_page() had severe failure.\n");
 		i_ret = 3; goto ex;
 	    }
-	    printf("Warning: test_rwr_mode_page() had minor failure.\n");
+	    printf("-- Warning: test_rwr_mode_page() had minor failure.\n");
 	}
-	
-	
+
+
 	if (with_tray_dance) {
 	    /* More surely provoke a non-trivial sense reply */
 	    if (test_cycle_with_media) {
@@ -644,15 +663,15 @@ test_write(char *psz_drive_path, unsigned int i_flag)
 		wait_for_drive(p_cdio, 15, 2 | b_verbose);
 	    }
 	}
-	
+
 	/* How are we, finally ? */
 	i_ret = test_unit_ready(p_cdio, &i_sense_valid, &sense_reply, b_verbose);
-	if ((i_flag & 1) && 0 != i_ret && 2 == sense_reply.sense_key && 
+	if ((i_flag & 1) && 0 != i_ret && 2 == sense_reply.sense_key &&
 	    0x3a == sense_reply.asc)
 	    fprintf(stderr, "test_unit_ready: Note: No loaded media detected.\n");
 	i_ret = 0;
     }
-    
+
   ex:;
     cdio_loglevel_default = old_log_level;
     cdio_destroy(p_cdio);
@@ -670,23 +689,23 @@ main(int argc, const char *argv[])
   int exitrc = 0;
   bool b_verbose = (argc > 1);
 
-  cdio_loglevel_default = b_verbose ? CDIO_LOG_DEBUG : CDIO_LOG_INFO;
+  cdio_loglevel_default = b_verbose ? CDIO_LOG_DEBUG : CDIO_LOG_WARN;
 
   ppsz_drives = cdio_get_devices(DRIVER_DEVICE);
   if (!ppsz_drives) {
     printf("Can't find a CD-ROM drive. Skipping test.\n");
     exit(SKIP_TEST);
   }
-  
+
   p_cdio = cdio_open(ppsz_drives[0], DRIVER_DEVICE);
   if (p_cdio) {
     const char *psz_have_mmc = cdio_get_arg(p_cdio, "mmc-supported?");
-    
-    if ( psz_have_mmc 
+
+    if ( psz_have_mmc
 	 && 0 == strncmp("true", psz_have_mmc, sizeof("true")) ) {
 
 	/* Test the MMC enhancements of version 0.83 in december 2009 */
-	ret = test_write(ppsz_drives[0], 
+	ret = test_write(ppsz_drives[0],
 			cdio_loglevel_default == CDIO_LOG_DEBUG);
 	if (ret != 0) exit(ret + 16);
     }
